/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007  Arto Karppinen (arto.karppinen@iki.fi),
 * Joni Kokko (t5kojo01@students.oamk.fi),
 * Sami Mäki (t5masa02@students.oamk.fi),
 * Harri Vattulainen (t5vaha01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/*
 * Ir Remote Control backend API.
 *
 * The idea is that one backend should handle one device type, and that there
 * could be multible instances of a device type in use.
 *
 * For example, the could be a backend for Lirc server, and there could be one
 * instance per each Lirc server the user wants to use.
 *
 * Each intance should contain a list of "devices" and each device should have
 * a list of "commands". 
 *
 * For example, if the lirk server has been configured with the the ir-codes
 * of two remotes the those two remotes would be the "devices", and the
 * ir-signals the remotes send would be the "commands".
 *
 * So the structure would be something like this:
 * 
 * Lirc Server               Lirc backend instance 1
 *     Yamaha amplifier      Lirc backend instance 1, Device 1
 *         Volume up         Lirc backend instance 1, Device 1, Command 1
 *         Volume down       Lirc backend instance 1, Device 1, Command 2
 *         Power             Lirc backend instance 1, Device 1, Command 3
 *     CD player             Lirc backend instance 1, Device 2
 *         Next track        Lirc backend instance 1, Device 2, Command 1
 *         Previous track    Lirc backend instance 1, Device 2, Command 2
 *         Play              Lirc backend instance 1, Device 2, Command 3
 * Lirc Server 2             Lirc backend instance 2
 *     Sony TV               Lirc backend instance 2, Device 1
 *         Power             Lirc backend instance 2, Device 1, Command 1
 *         Previous channel  Lirc backend instance 2, Device 1, Command 2
 *         Next channel      Lirc backend instance 2, Device 1, Command 3   
 *
 * .... and so on until the world runs out of Lirc servers ....
 *
 * In the user interface backend instance is called device controller. Which
 * hopefully makes more sense to the user.
 */
#ifndef __IRRECO_BACKEND_API__
#define __IRRECO_BACKEND_API__

#include <glib.h>
#include <gtk/gtk.h>
#include "irreco_backend_file_container.h"


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Implemented by IRRECO.                                                        */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Get device callback. The backend should call this once per device.
 *
 * Name is the name which will be displayed to the user.
 * Contex is whatever the backend should need to use/identify the device.
 * Irreco will make a copy of the name buffer.
 *
 * void irreco_get_device_callback(const gchar * name,
 * 				   gpointer device_contex)
 */
typedef void (*IrrecoGetDeviceCallback) (const gchar * name,
					 gpointer device_contex);


/**
 * Get command callback. The backend should call this once per command.
 *
 * Name is the name which will be displayed to the user.
 * Contex is whatever the backend should need to use/identify the command.
 * Irreco will make a copy of the name buffer.
 *
 * void irreco_get_command_callback(const gchar * name, 
 * 				    gpointer command_contex);
 */
typedef void (*IrrecoGetCommandCallback) (const gchar * name, 
					  gpointer command_contex);



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Error handling.                                                            */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Most backend functions must return a status code. If this status code is
 * IRRECO_BACKEND_OK then Irreco assumes that the function succeeded in 
 * whatever it was doing.
 *
 * However, if the status code is anything else than IRRECO_BACKEND_OK then
 * Irreco will assume that the function failed, and asks for a more descriptive 
 * error message with IrrecoBackendGetErrorMsg. Then the error message and 
 * the code will be displayed to the user.
 */
typedef enum {
	IRRECO_BACKEND_OK = 0
} IrrecoBackendStatus;



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* To be Implemented by Backend.                                              */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Return error message which corresponds to the backend status code.
 *
 * const gchar *BACKEND_NAME_get_error_msg(gpointer instance_context, 
 *					  IrrecoBackendStatus code);
 */
typedef const gchar *(*IrrecoBackendGetErrorMsg) (gpointer instance_context, 
						 IrrecoBackendStatus code);

/**
 * Create new instance of the backend.
 *
 * The backend should be able to create multiple instances of itself.
 * Returns a new instance_context of the backend.
 *
 * gpointer BACKEND_NAME_create()
 */
typedef gpointer(*IrrecoBackendCreate) ();


/**
 * Destroy instance of the backend. 
 *
 * If 'permanently' argument is FALSE, the backend instance is to be reloaded 
 * from the configuration file the next time Irreco is started. If 'permanently' 
 * is TRUE, the configuration file will be deleted by Irreco, and this instace 
 * will be destroyd permanently.
 *
 * If Irreco is closed, and backend instace is destroyed, then permanently is 
 * FALSE. If user chooses to delete controller, then permanently is TRUE.
 *
 * void BACKEND_NAME_destroy(gpointer instance_context)
 */
typedef void (*IrrecoBackendDestroy) (gpointer instance_context, 
				      gboolean permanently);


/**
 * Create new instance of the backend.
 *
 * The backend should be able to read the configuration file and configure 
 * itself accordingly.
 *
 * IrrecoBackendStatus BACKEND_NAME_read_from_conf(gpointer instance_context, 
 *						   const gchar * config_file)
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendReadFromConf) (gpointer instance_context, 
			      const gchar * config_file);


/**
 * Save backend configuration into the file.
 *
 * Irreco will assign the filename to use.
 *
 * IrrecoBackendStatus BACKEND_NAME_save_to_conf(gpointer instance_context, 
 *						 const gchar * config_file)
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendSaveToConf) (gpointer instance_context, 
			    const gchar * config_file);


/**
 * Get list of devices the current backend instance knows.
 *
 * void BACKEND_NAME_get_devices(gpointer instance_context, 
 *				 IrrecoGetDeviceCallback callback)
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendGetDevices) (gpointer instance_context,
			    IrrecoGetDeviceCallback callback);


/**
 * Get list of commands the device supports.
 *
 * IrrecoBackendStatus 
 * BACKEND_NAME_get_commands(gpointer instance_context,
 *			     const gchar * device_name,
 *			     gpointer device_contex,
 *			     IrrecoGetCommandCallback callback);
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendGetCommands) (gpointer instance_context,
			     const gchar * device_name,
			     gpointer device_contex,
			     IrrecoGetCommandCallback callback);


/**
 * Send command to device.
 *
 * IrrecoBackendStatus BACKEND_NAME_send_command(gpointer instance_context,
 *						 const gchar * device_name,
 *						 gpointer device_contex,
 *						 const gchar * command_name,
 *						 gpointer command_contex);
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendSendCommand) (gpointer instance_context, 
			     const gchar * device_name,
			     gpointer device_contex, 
			     const gchar * command_name,
			     gpointer command_contex);


/**
 * Configure the backend. The backend should pop up some kind of dialog where
 * the user can input the configuration.
 * 
 * IrrecoBackendStatus BACKEND_NAME_configure(gpointer instance_context)
 */
typedef IrrecoBackendStatus (*IrrecoBackendConfigure) (gpointer instance_context,
						       GtkWindow * parent);



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Optionally Implemented by Backend.                                         */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Get a description of the instace. 
 *
 * The description should be about 10 - 30 characters long string that 
 * differentiates one instance from all the other instances. The description 
 * is to be displayed to the user.
 *
 * @return newly-allocated string.
 * @code
 * gchar *BACKEND_NAME_get_description(gpointer instance_context);
 * @endcode
 */
typedef gchar *(*IrrecoBackendGetDescription) (gpointer instance_context);


/**
 * Create a new device which can be controlled.
 *
 * The backend is expected to show appropriate dialog if needed.
 *
 * @return IRRECO_BACKEND_OK or error code.
 * @code
 * IrrecoBackendStatus BACKEND_NAME_create_device(gpointer instance_context, 
 *						  GtkWindow * parent);
 * @endcode
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendCreateDevice) (gpointer instance_context, 
			      GtkWindow * parent);

/**
 * Can the device be edited?
 * 
 * Irreco uses this to determinate if the device can be edited. This is usefull
 * mainly if the backend supports both editable and uneditable devices.
 *
 * @return TRUE or FALSE
 * @code
 * gboolean BACKEND_NAME_is_device_editable(gpointer instance_context, 
 *					    const gchar * device_name,
 *					    gpointer device_contex);
 * @endcode
 */
typedef gboolean (*IrrecoBackendIsDeviceEditable) (gpointer instance_context, 
						   const gchar * device_name,
						   gpointer device_contex);

/**
 * Edit currently existing device.
 *
 * Basically this means, that the user should be able to add, remove and edit
 * commands the device has. The backend is expected to show appropriate dialog 
 * if needed.
 *
 * @return IRRECO_BACKEND_OK or error code.
 * @code
 * IrrecoBackendStatus BACKEND_NAME_edit_device(gpointer instance_context, 
 *						const gchar * device_name,
 *						gpointer device_contex, 
 *						GtkWindow * parent);
 * @endcode
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendEditDevice) (gpointer instance_context, 
			    const gchar * device_name, 
			    gpointer device_contex, 
			    GtkWindow * parent);

/**
 * Destroy a device.
 *
 * The backend is expected to show appropriate dialog if needed.
 *
 * @return IRRECO_BACKEND_OK or error code.
 * @code
 * IrrecoBackendStatus BACKEND_NAME_delete_device(gpointer instance_context, 
 *						  const gchar * device_name,
 *						  gpointer device_contex, 
 *						  GtkWindow * parent);
 * @endcode
 */
typedef IrrecoBackendStatus 
(*IrrecoBackendDeleteDevice) (gpointer instance_context, 
			      const gchar * device_name,
			      gpointer device_contex, 
			      GtkWindow * parent);

/**
 * Get configuration of backend.
 *
 * Returned filecontainer should include hash, name and data.
 *
 * @return IRRECO_BACKEND_OK or error code.
 * @code
 * IrrecoBackendStatus BACKEND_NAME_export_conf(gpointer instance_context,
				const char * device_name,
 *				IrrecoBackendFileContainer **file_container);
 *
 * @endcode
 */
typedef IrrecoBackendStatus (*IrrecoBackendExportConf) (
				gpointer instance_context,
    				const char * device_name,
				IrrecoBackendFileContainer **file_container);

/**
 * Set configuration of backend.
 *
 * Set filecontainer should include hash, name and data.
 *
 * @return IRRECO_BACKEND_OK or error code.
 * @code
 * IrrecoBackendStatus BACKEND_NAME_import_conf(gpointer instance_context,
 *				IrrecoBackendFileContainer *file_container);
 *
 * @endcode
 */
typedef IrrecoBackendStatus (*IrrecoBackendImportConf) (
				gpointer instance_context,
				IrrecoBackendFileContainer *file_container);

/**
 * Is there already configuration like this?
 *
 * Set filecontainer should include hash, name and data, etc.
 *
 * @return IRRECO_BACKEND_OK or error code.
 * @code
 * IrrecoBackendStatus BACKEND_NAME_check_conf(
 *				gpointer instance_context,
 *				IrrecoBackendFileContainer *file_container,
				gboolean *configuration);
 * @endcode
 */
typedef IrrecoBackendStatus (*IrrecoBackendCheckConf) 
				(gpointer instance_context,
				IrrecoBackendFileContainer *file_container,
				gboolean *configuration);


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Function table structure.                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

typedef enum {
	/**
	 * 
	 */
	IRRECO_BACKEND_EDITABLE_DEVICES		= 1 << 1,
 	IRRECO_BACKEND_CONFIGURATION_EXPORT	= 1 << 2,
  	IRRECO_BACKEND_MULTI_DEVICE_SUPPORT	= 1 << 3,
   	IRRECO_BACKEND_NEW_INST_ON_CONF_IMPORT	= 1 << 4

} IrrecoBackendFlags;

#define IRRECO_BACKEND_API_VERSION 3
typedef struct _IrrecoBackendFunctionTable IrrecoBackendFunctionTable;
struct _IrrecoBackendFunctionTable {
	guint				version;
	IrrecoBackendFlags		flags;
	const gchar *			name;
	IrrecoBackendGetErrorMsg	get_error_msg;
	IrrecoBackendCreate 		create;
	IrrecoBackendDestroy		destroy;
	IrrecoBackendReadFromConf	from_conf;	
	IrrecoBackendSaveToConf		to_conf;
	IrrecoBackendGetDevices		get_devices;
	IrrecoBackendGetCommands	get_commands;
	IrrecoBackendSendCommand	send_command;
	IrrecoBackendConfigure		configure;
	
	IrrecoBackendGetDescription	get_description;
	IrrecoBackendCreateDevice	create_device;
	IrrecoBackendIsDeviceEditable	is_device_editable;
	IrrecoBackendEditDevice		edit_device;
	IrrecoBackendDeleteDevice	delete_device;
	
	IrrecoBackendExportConf		export_conf;
	IrrecoBackendImportConf		import_conf;
	IrrecoBackendCheckConf		check_conf;
	
	gpointer			reserved4;
};


/*
typedef enum {
	IRRECO_BACKEND_EDITABLE_DEVICES = 1
} IrrecoBackendFlags;

#define IRRECO_BACKEND_API_VERSION 2
typedef struct _IrrecoBackendFunctionTable IrrecoBackendFunctionTable;
struct _IrrecoBackendFunctionTable {
	guint				version;
	IrrecoBackendFlags		flags;
	const gchar *			name;
	IrrecoBackendGetErrorMsg	get_error_msg;
	IrrecoBackendCreate 		create;
	IrrecoBackendDestroy		destroy;
	IrrecoBackendReadFromConf	from_conf;	
	IrrecoBackendSaveToConf		to_conf;
	IrrecoBackendGetDevices		get_devices;
	IrrecoBackendGetCommands	get_commands;
	IrrecoBackendSendCommand	send_command;
	IrrecoBackendConfigure		configure;
	
	IrrecoBackendGetDescription	get_description;
	IrrecoBackendCreateDevice	create_device;
	IrrecoBackendIsDeviceEditable	is_device_editable;
	IrrecoBackendEditDevice		edit_device;
	IrrecoBackendDeleteDevice	delete_device;
	
	gpointer			reserved1;
	gpointer			reserved2;
	gpointer			reserved3;
	gpointer			reserved4;
};
*/

/**
 * Get function table of the backend.
 *
 * GetIrrecoBackendFunctionTable() and IrrecoBackendFunctionTable
 * should be implemented like this:
 *
 * @code
 * IrrecoBackendFunctionTable BACKEND_NAME_function_table = {
 *	IRRECO_BACKEND_API_VERSION,	/_ backend_api_version		_/
 *	0,				/_ flags 			_/
 *	"BACKEND_NAME",			/_ name				_/
 *
 *	BACKEND_NAME_get_error_msg,	/_ get_error_msg 		_/
 *	BACKEND_NAME_create,		/_ create			_/
 *	BACKEND_NAME_destroy,		/_ destroy			_/
 *	BACKEND_NAME_read_from_conf,	/_ from_conf			_/
 *	BACKEND_NAME_save_to_conf,	/_ to_conf			_/
 *	BACKEND_NAME_get_devices,	/_ get_devices			_/
 *	BACKEND_NAME_get_commands,	/_ get_commands			_/
 *	BACKEND_NAME_send_command,	/_ send_command			_/
 *	BACKEND_NAME_configure		/_ configure			_/   
 *	
 *	NULL,				/_ get_description,    optional	_/ 
 *	NULL,				/_ create_device,      optional	_/ 
 *	NULL,				/_ is_device_editable, optional	_/ 
 *	NULL,				/_ edit_device,        optional	_/ 
 *	NULL,				/_ delete_device,      optional	_/ 
 *	
 *	NULL, NULL, NULL, NULL		/_ reserved _/   
 * };
 * IrrecoBackendFunctionTable *get_irreco_backend_function_table() {
 * 	return &BACKEND_NAME_function_table;
 * }
 * @endcode
 */
typedef IrrecoBackendFunctionTable *(*GetIrrecoBackendFunctionTable) ();



#endif /* __IRRECO_BACKEND_API__ */

