/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007  Arto Karppinen (arto.karppinen@iki.fi)
 *	      &  2008  Joni Kokko     (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_device_dlg.h"
#include "irreco_backend_device.h"
#include "irreco_select_instance_dlg.h"
#include "irreco_webdb_upload_dlg.h"


G_DEFINE_TYPE (IrrecoDeviceDlg, irreco_device_dlg, IRRECO_TYPE_DLG)
		
/*
 * IrrecoDeviceDlg
 *
 * Dialog for creating, deleting and editing backend devices.
 */


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Datatypes                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

enum {
IRRECO_DEVICE_NEW,
IRRECO_DEVICE_EDIT,
IRRECO_DEVICE_DELETE,
IRRECO_UPLOAD_DEVICE,
IRRECO_DEVICE_REFERESH,
IRRECO_DEVICE_OK
};

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

static void irreco_device_dlg_populate(IrrecoDeviceDlg * device_dlg);
static void irreco_device_dlg_selection_changed(GtkTreeSelection * selection, 
					 IrrecoDeviceDlg * device_dlg);

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

static void irreco_device_dlg_dispose (GObject *object)
{
  if (G_OBJECT_CLASS (irreco_device_dlg_parent_class)->dispose)
    G_OBJECT_CLASS (irreco_device_dlg_parent_class)->dispose (object);
}

static void irreco_device_dlg_finalize (GObject *object)
{
  if (G_OBJECT_CLASS (irreco_device_dlg_parent_class)->finalize)
    G_OBJECT_CLASS (irreco_device_dlg_parent_class)->finalize (object);
}

static void irreco_device_dlg_class_init (IrrecoDeviceDlgClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  object_class->dispose = irreco_device_dlg_dispose;
  object_class->finalize = irreco_device_dlg_finalize;
}

static void irreco_device_dlg_init (IrrecoDeviceDlg *self)
{
	GtkWidget *padding;
	GtkTreeSelection *select;
	IRRECO_ENTER

	/* Construct dialog. */
	gtk_window_set_title(GTK_WINDOW(self), _("Devices"));
	gtk_window_set_modal(GTK_WINDOW(self), TRUE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(self), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(self), FALSE);
	gtk_dialog_add_buttons(GTK_DIALOG(self), 
			       GTK_STOCK_NEW, IRRECO_DEVICE_NEW,
		               GTK_STOCK_EDIT, IRRECO_DEVICE_EDIT,
		               GTK_STOCK_DELETE, IRRECO_DEVICE_DELETE,
		 	       _("Upload"), IRRECO_UPLOAD_DEVICE,
		               GTK_STOCK_REFRESH, IRRECO_DEVICE_REFERESH,
		               GTK_STOCK_OK, IRRECO_DEVICE_OK,
			       NULL);
	
	/* Add widgets. */
	self->vbox = gtk_vbox_new(FALSE, 0);
	self->listbox = IRRECO_LISTBOX(
			irreco_listbox_text_new_with_autosize(0, 700, 100, 250));
	
	gtk_container_add(GTK_CONTAINER(self->vbox), GTK_WIDGET(self->listbox));
	padding = irreco_gtk_pad(self->vbox, 8, 8, 8, 8);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(self)->vbox), padding);

	/* Selection signal. */
	select = gtk_tree_view_get_selection(
			 GTK_TREE_VIEW(self->listbox->tree_view));
	g_signal_connect(G_OBJECT(select), "changed",
			 G_CALLBACK(irreco_device_dlg_selection_changed), 
			 self);
			
	IRRECO_RETURN
}

GtkWidget * irreco_device_dlg_new (GtkWindow *parent)
{
	IrrecoDeviceDlg *self;
	IRRECO_ENTER
			
	self = g_object_new(IRRECO_TYPE_DEVICE_DLG, NULL);
	irreco_dlg_set_parent(IRRECO_DLG(self), parent);
	
	IRRECO_RETURN_PTR(self);
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*
 * Populate listbox with devices.
 */
static void irreco_device_dlg_populate(IrrecoDeviceDlg * device_dlg)
{
	IRRECO_ENTER
	irreco_listbox_clear(device_dlg->listbox);
	IRRECO_BACKEND_MANAGER_FOREACH_INSTANCE(
	    device_dlg->irreco_data->irreco_backend_manager, instance)
		irreco_backend_instance_get_name_and_description(instance);
		IRRECO_BACKEND_INSTANCE_FOREACH(instance, device)
			GString *str = g_string_new(NULL);
			g_string_append_printf(str, "%s - %s",
				instance->name_and_description, device->name);
			IRRECO_DEBUG("Adding \"%s\" to listbox.\n", str->str);
			irreco_listbox_text_append(IRRECO_LISTBOX_TEXT(
						   device_dlg->listbox), 
						   str->str, device);
			g_string_free(str, TRUE);
		IRRECO_BACKEND_INSTANCE_FOREACH_END
	IRRECO_BACKEND_MANAGER_FOREACH_END
	
	IRRECO_RETURN
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Functions                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_show_device_dlg(IrrecoData *irreco_data,
			    GtkWindow *parent)
{
	IrrecoDeviceDlg *device_dlg;
	gint sel_index;
	gpointer sel_user_data;
	IrrecoBackendDevice *device;
	gboolean loop = TRUE;
	IRRECO_ENTER
			
	device_dlg = IRRECO_DEVICE_DLG(irreco_device_dlg_new(parent));
	
	device_dlg->irreco_data = irreco_data;

	irreco_device_dlg_populate(device_dlg);
	
	gtk_widget_show_all(GTK_WIDGET(device_dlg));
	while (loop == TRUE)
	{
		switch (gtk_dialog_run(GTK_DIALOG(device_dlg))) {
		case IRRECO_DEVICE_NEW: {
			IrrecoBackendInstance *instance = NULL;
			IRRECO_DEBUG("IRRECO_DEVICE_NEW\n");
			if (irreco_show_select_instance_dlg(
			    device_dlg->irreco_data, GTK_WINDOW(device_dlg),
			    NULL, &instance)) {
				irreco_backend_instance_create_device(
				instance, GTK_WINDOW(device_dlg));
				irreco_backend_instance_get_devcmd_list(
					instance);
				irreco_device_dlg_populate(device_dlg);
			}
			} break;
			
		case IRRECO_DEVICE_DELETE: {
			IRRECO_DEBUG("IRRECO_DEVICE_DELETE\n");
			irreco_listbox_get_selection(
				IRRECO_LISTBOX(device_dlg->listbox), 
				&sel_index, NULL, &sel_user_data);
			device = (IrrecoBackendDevice *) sel_user_data;
			if (device == NULL) break;
			
			if (!irreco_backend_device_is_editable(device)) break;
			
			irreco_backend_device_delete(
				device, GTK_WINDOW(device_dlg));
			irreco_backend_instance_get_devcmd_list(
				device->backend_instance);
			irreco_device_dlg_populate(device_dlg);
			} break;
			
		case IRRECO_UPLOAD_DEVICE: {
			IrrecoBackendFileContainer *file_container = NULL;
			IRRECO_DEBUG("IRRECO_UPLOAD_DEVICE\n");
			irreco_listbox_get_selection(
				IRRECO_LISTBOX(device_dlg->listbox), 
				&sel_index, NULL, &sel_user_data);
			device = (IrrecoBackendDevice *) sel_user_data;
			if (device == NULL) break;		
			
			if (irreco_backend_instance_export_conf(
						device->backend_instance,
      						device->name,
						&file_container)) {

				irreco_show_webdb_upload_dlg(
							device_dlg->irreco_data,
							GTK_WINDOW(device_dlg),
							file_container);
				irreco_backend_file_container_free(
							file_container);
			}

			break;
		}
			
		case IRRECO_DEVICE_EDIT: {
			IRRECO_DEBUG("IRRECO_DEVICE_EDIT\n");
			irreco_listbox_get_selection(
				IRRECO_LISTBOX(device_dlg->listbox), 
				&sel_index, NULL, &sel_user_data);
			device = (IrrecoBackendDevice *) sel_user_data;
			if (device == NULL) break;
			
			if (!irreco_backend_device_is_editable(device)) break;
			
			irreco_backend_device_edit(
				device, GTK_WINDOW(device_dlg));

			irreco_backend_instance_save_to_conf(device->backend_instance);
			
			irreco_backend_instance_get_devcmd_list(
				device->backend_instance);
			irreco_device_dlg_populate(device_dlg);
			} break;
			
		
		case IRRECO_DEVICE_REFERESH:
			irreco_backend_manager_get_devcmd_lists(
			       device_dlg->irreco_data->irreco_backend_manager);
			irreco_device_dlg_populate(device_dlg);
			break;
			
		case IRRECO_DEVICE_OK:
			IRRECO_DEBUG("IRRECO_DEVICE_OK\n");
			loop = FALSE;
			break;
		}
		
		gtk_widget_grab_focus(device_dlg->listbox->tree_view);

	}
	gtk_widget_destroy(GTK_WIDGET(device_dlg));
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/* 
 * Enable / disable buttons depending on editability.
 */
static void irreco_device_dlg_selection_changed(GtkTreeSelection * selection, 
					 	IrrecoDeviceDlg * device_dlg)
{
	gint sel_index;
	gpointer sel_user_data;
	IrrecoBackendDevice *device;
	GtkWidget *edit_button;
	GtkWidget *delete_button;
	GtkWidget *upload_button;
	IRRECO_ENTER
	
	irreco_listbox_get_selection(IRRECO_LISTBOX(device_dlg->listbox), 
				     &sel_index, NULL, &sel_user_data);
	if (sel_user_data == NULL) IRRECO_RETURN
	
	edit_button = irreco_gtk_dialog_get_button(GTK_WIDGET(device_dlg), 1);
	delete_button = irreco_gtk_dialog_get_button(GTK_WIDGET(device_dlg), 2);
	upload_button = irreco_gtk_dialog_get_button(GTK_WIDGET(device_dlg), 3);
	
	device = (IrrecoBackendDevice *) sel_user_data;
	if (irreco_backend_device_is_editable(device)) {
		gtk_widget_set_sensitive(edit_button, TRUE);
		gtk_widget_set_sensitive(delete_button, TRUE);
	} else {
		gtk_widget_set_sensitive(edit_button, FALSE);
		gtk_widget_set_sensitive(delete_button, FALSE);
	}

	if (device->backend_instance->lib->api->flags &
	    IRRECO_BACKEND_CONFIGURATION_EXPORT) {
		gtk_widget_set_sensitive(upload_button, TRUE);
	} else {
		gtk_widget_set_sensitive(upload_button, FALSE);
	}

	IRRECO_RETURN
}

