/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007  Arto Karppinen (arto.karppinen@iki.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_style_browser_dlg.h"
#include "irreco_scrolled_window.h"
#include <hildon/hildon-banner.h>

/**
 * @typedef IrrecoStyleBrowser
 *
 * Dialog which displays a list of button styles to the user, so the user
 * can select one.
 */
 
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
static gboolean 
irreco_style_browser_dlg_loading_loop(IrrecoStyleBrowserDlgLoading *loading);
static void irreco_style_browser_dlg_button_click(IrrecoButton *irreco_button, 
						  IrrecoStyleBrowserDlg *self);
static gboolean irreco_style_browser_dlg_map_event(IrrecoStyleBrowserDlg *self,
						   GdkEvent *event,
						   gpointer user_data);
/*static gboolean 
irreco_style_browser_dlg_configure_event(IrrecoStyleBrowserDlg *self,
					 GdkEventConfigure *event,
					 gpointer user_data);*/



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

G_DEFINE_TYPE(IrrecoStyleBrowserDlg, irreco_style_browser_dlg, IRRECO_TYPE_DLG)

static void irreco_style_browser_dlg_finalize(GObject *object)
{
	IRRECO_ENTER
	G_OBJECT_CLASS(irreco_style_browser_dlg_parent_class)->finalize(object);
	g_hash_table_destroy(IRRECO_STYLE_BROWSER_DLG(object)->layout_array);
	IRRECO_RETURN
}

static void irreco_style_browser_dlg_class_init(IrrecoStyleBrowserDlgClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);
	object_class->finalize = irreco_style_browser_dlg_finalize;
}

static void irreco_style_browser_dlg_init(IrrecoStyleBrowserDlg *self)
{
	GtkWidget *scrolled;
	IRRECO_ENTER
	
	/* Create array for irreco layouts. */
	self->layout_array = g_hash_table_new_full(
		g_direct_hash, g_direct_equal, NULL, 
		G_DESTROYNOTIFY(irreco_button_layout_destroy));
	
	/* Construct dialog. */
	gtk_window_set_title(GTK_WINDOW(self), _("Style Browser"));
	gtk_window_set_modal(GTK_WINDOW(self), TRUE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(self), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(self), FALSE);
	gtk_dialog_add_buttons(GTK_DIALOG(self),
			       GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, 
			       NULL);
	
	/* Create widgets. */
	scrolled = irreco_scrolled_window_new_with_autoresize(300, 700, 
							      200, 300);
	self->vbox = gtk_vbox_new(FALSE, 10);
	
	/* Add widgets to dialog. */
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(self)->vbox), scrolled);
	irreco_scrolled_window_add(IRRECO_SCROLLED_WINDOW(scrolled), 
				   self->vbox);
	
	/* Signal handlers. */
	g_signal_connect(G_OBJECT(self), "map-event",
			 G_CALLBACK(irreco_style_browser_dlg_map_event), NULL);
	/*g_signal_connect(G_OBJECT(self), "configure-event",
			 G_CALLBACK(irreco_style_browser_dlg_configure_event),
			 NULL);*/
			 
	/* Show dialog, hopefully on the left-top corner of the screen. */
	gtk_widget_show_all(GTK_WIDGET(self));
	gtk_window_move(GTK_WINDOW(self), 80, 60);
	IRRECO_RETURN
}

GtkWidget* irreco_style_browser_dlg_new(IrrecoData *irreco_data, 
					GtkWindow *parent)
{
	IrrecoStyleBrowserDlg *self;
	IRRECO_ENTER
	
	self = g_object_new(IRRECO_TYPE_STYLE_BROWSER_DLG, NULL);
	irreco_dlg_set_parent(IRRECO_DLG(self), parent);
	irreco_style_browser_dlg_set_irreco_data(self, irreco_data);
	IRRECO_RETURN_PTR(self);
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

static void irreco_style_browser_dlg_add_style(IrrecoStyleBrowserDlg *self,
					       IrrecoButtonStyle *style)
{
	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *fixed;                                          
	IrrecoButtonLayout *layout;
	IRRECO_ENTER
	
	/* Create widgets. */
	if (style == NULL) {
		label = gtk_label_new(_("Normal button"));
	} else {
		label = gtk_label_new(style->name);
	}
	hbox = gtk_hbox_new(FALSE, 10);
	fixed = gtk_fixed_new();

	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), fixed, 0, 0, 0);
	gtk_box_pack_end_defaults(GTK_BOX(hbox), label);
	gtk_container_add(GTK_CONTAINER(self->vbox), hbox);
	gtk_widget_show_all(hbox);
	
	/* Create layout and setup callbacks. */
	layout = irreco_button_layout_create(fixed, NULL);
	g_hash_table_insert(self->layout_array, layout, layout);
	irreco_button_layout_set_press_callback(layout, irreco_button_down);
	irreco_button_layout_set_release_callback(
		layout, irreco_style_browser_dlg_button_click);
	irreco_button_layout_set_callback_data(layout, self);

	/* Create button. */
	irreco_button_create(layout, 0, 0, _("Select"), style, NULL);
	irreco_button_layout_create_widgets(layout);

	IRRECO_RETURN
}

static void irreco_style_browser_dlg_loading_start(IrrecoStyleBrowserDlg *self)
{
	IRRECO_ENTER
			
	if (self->loading == NULL) {
		self->loading = g_slice_new0(IrrecoStyleBrowserDlgLoading);
		self->loading->self = self;
		self->loading->len = irreco_string_table_lenght(
			self->irreco_data->button_style_array);
		self->loading->banner = hildon_banner_show_progress(
			GTK_WIDGET(self), NULL, "Loading");
		hildon_banner_set_fraction(HILDON_BANNER(self->loading->banner), 
			(float) 1 / (float) self->loading->len);
		g_hash_table_remove_all(self->layout_array);
		
		/* Add GtkButton layout. */
		irreco_style_browser_dlg_add_style(self, NULL);
		g_idle_add(G_SOURCEFUNC(irreco_style_browser_dlg_loading_loop),
			   self->loading);
	}
	IRRECO_RETURN
}

static void irreco_style_browser_dlg_loading_cancel(IrrecoStyleBrowserDlg *self)
{
	IRRECO_ENTER
	if (self->loading != NULL) self->loading->self = NULL;
	IRRECO_RETURN
}

static gboolean 
irreco_style_browser_dlg_loading_loop(IrrecoStyleBrowserDlgLoading *loading)
{
	const gchar *key;
	gpointer style;
	IRRECO_ENTER
	
	/* Use has closed the dialog before the loading has been completed. 
	   When we hit this, Gtk has already destroyed the dialog, so all we 
	   need to do is to realease the loading strucure.*/
	if (loading->self == NULL) {
		IRRECO_DEBUG("Cancelling style loading.\n");
		g_slice_free(IrrecoStyleBrowserDlgLoading, loading);
		IRRECO_RETURN_BOOL(FALSE); 
		
	} else if (irreco_string_table_index(
			loading->self->irreco_data->button_style_array, 
			loading->pos, &key, &style)) {
		IRRECO_DEBUG("Loading style \"%s\".\n", key);
		irreco_style_browser_dlg_add_style(loading->self, 
						   (IrrecoButtonStyle *) style);
		loading->pos = loading->pos + 1;
		hildon_banner_set_fraction(HILDON_BANNER(loading->banner), 
			(float) 1 / (float) loading->len 
			* (float) loading->pos);
		IRRECO_RETURN_BOOL(TRUE); 
		
	} else {
		IRRECO_DEBUG("Loading complete. Cleaning up.\n");
		gtk_widget_destroy(loading->banner);
		loading->self->loading = NULL;
		loading->banner = NULL;
		loading->pos = 0;
		g_slice_free(IrrecoStyleBrowserDlgLoading, loading);
		IRRECO_RETURN_BOOL(FALSE); 
	}
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Functions                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_style_browser_dlg_set_irreco_data(IrrecoStyleBrowserDlg *self,
					      IrrecoData *irreco_data)
{
	IRRECO_ENTER
	self->irreco_data = irreco_data;
	IRRECO_RETURN
}

gboolean irreco_show_style_browser_dlg(IrrecoData *irreco_data,
				       GtkWindow *parent,
				       IrrecoButtonStyle **style)
{
	gint response;
	IrrecoStyleBrowserDlg *self;
	IRRECO_ENTER
	
	self = IRRECO_STYLE_BROWSER_DLG(irreco_style_browser_dlg_new(
					irreco_data, parent));	
	response = gtk_dialog_run(GTK_DIALOG(self));
	irreco_style_browser_dlg_loading_cancel(self);

	if (response == 1) *style = self->style;
	gtk_widget_destroy(GTK_WIDGET(self));
	
	if (response == 1) IRRECO_RETURN_BOOL(TRUE);
	IRRECO_RETURN_BOOL(FALSE);
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

static void irreco_style_browser_dlg_button_click(IrrecoButton *irreco_button, 
						  IrrecoStyleBrowserDlg *self)
{
	IRRECO_ENTER
	self->style = irreco_button->style;
	gtk_dialog_response(GTK_DIALOG(self), 1);
	IRRECO_RETURN
}

static gboolean irreco_style_browser_dlg_map_event(IrrecoStyleBrowserDlg *self,
						   GdkEvent *event,
						   gpointer user_data)
{
	IRRECO_ENTER
	irreco_style_browser_dlg_loading_start(self);
	IRRECO_RETURN_BOOL(FALSE);
}

/*
static gboolean 
irreco_style_browser_dlg_configure_event(IrrecoStyleBrowserDlg *self,
					 GdkEventConfigure *event,
					 gpointer user_data) 
{
	IRRECO_ENTER
	IRRECO_PRINTF("Dialog position x%i y%i\n", event->x, event->y);
	IRRECO_RETURN_BOOL(FALSE);
}
*/












